@@ -99,10 +99,6 @@ extern int clock_nanosleep(clockid_t __clock_id, int __flags,
 /* Must be power of 2 ! */
 #define VALBUF_SIZE		16384
 
-#define KVARS			32
-#define KVARNAMELEN		32
-#define KVALUELEN		32
-
 #if (defined(__i386__) || defined(__x86_64__))
 #define ARCH_HAS_SMI_COUNTER
 #endif
@@ -194,6 +190,7 @@ static int ct_debug;
 static int use_fifo = 0;
 static pthread_t fifo_threadid;
 static int laptop = 0;
+static int power_management = 0;
 static int use_histfile = 0;
 
 #ifdef ARCH_HAS_SMI_COUNTER
@@ -256,6 +253,11 @@ static void set_latency_target(void)
 		return;
 	}
 
+	if (power_management) {
+		warn("not setting cpu_dma_latency from cyclictest\n");
+		return;
+	}
+
 	errno = 0;
 	err = stat("/dev/cpu_dma_latency", &s);
 	if (err == -1) {
@@ -430,39 +432,81 @@ static int has_smi_counter(void)
 
 	model = (((fms >> 16) & 0xf) << 4) + ((fms >> 4) & 0xf);
 
+	/* Based on intel_model_duplicates */
 	switch (model) {
-	case 0x1A:      /* Core i7, Xeon 5500 series - Bloomfield, Gainstown NHM-EP */
-	case 0x1E:      /* Core i7 and i5 Processor - Clarksfield, Lynnfield, Jasper Forest */
+	case 0x1A:      /* INTEL_FAM6_NEHALEM_EP */
+	case 0x1E:      /* INTEL_FAM6_NEHALEM */
 	case 0x1F:      /* Core i7 and i5 Processor - Nehalem */
-	case 0x25:      /* Westmere Client - Clarkdale, Arrandale */
-	case 0x2C:      /* Westmere EP - Gulftown */
-	case 0x2E:      /* Nehalem-EX Xeon - Beckton */
-	case 0x2F:      /* Westmere-EX Xeon - Eagleton */
-	case 0x2A:      /* SNB */
-	case 0x2D:      /* SNB Xeon */
-	case 0x3A:      /* IVB */
-	case 0x3E:      /* IVB Xeon */
-	case 0x3C:      /* HSW */
-	case 0x3F:      /* HSX */
-	case 0x45:      /* HSW */
-	case 0x46:      /* HSW */
-	case 0x3D:      /* BDW */
-	case 0x47:      /* BDW */
-	case 0x4F:      /* BDX */
-	case 0x56:      /* BDX-DE */
-	case 0x4E:      /* SKL */
-	case 0x5E:      /* SKL */
-	case 0x8E:      /* KBL */
-	case 0x9E:      /* KBL */
-	case 0x55:      /* SKX */
-	case 0x37:      /* BYT */
-	case 0x4D:      /* AVN */
-	case 0x4C:      /* AMT */
-	case 0x57:      /* PHI */
-	case 0x5C:      /* BXT */
-	case 0x5F:	/* DNV */
-	case 0x7A:	/* Gemini Lake */
-	case 0x85:	/* Knights Mill */
+	case 0x25:      /* INTEL_FAM6_WESTMERE */
+	case 0x2C:      /* INTEL_FAM6_WESTMERE_EP */
+		model = 0x1E;      /* INTEL_FAM6_NEHALEM */
+		break;
+	case 0x2E:      /* INTEL_FAM6_NEHALEM_EX */
+	case 0x2F:      /* INTEL_FAM6_WESTMERE_EX */
+		model = 0x2E;      /* INTEL_FAM6_NEHALEM_EX */
+		break;
+	case 0x85:      /* INTEL_FAM6_XEON_PHI_KNM */
+		model = 0x57;      /* INTEL_FAM6_XEON_PHI_KNL */
+		break;
+	case 0x4F:      /* INTEL_FAM6_BROADWELL_X */
+	case 0x56:      /* INTEL_FAM6_BROADWELL_D */
+		model = 0x4F;      /* INTEL_FAM6_BROADWELL_X */
+		break;
+	case 0x4E:      /* INTEL_FAM6_SKYLAKE_L */
+	case 0x5E:      /* INTEL_FAM6_SKYLAKE */
+	case 0x8E:      /* INTEL_FAM6_KABYLAKE_L */
+	case 0x9E:      /* INTEL_FAM6_KABYLAKE */
+	case 0xA6:      /* INTEL_FAM6_COMETLAKE_L */
+	case 0xA5:      /* INTEL_FAM6_COMETLAKE */
+		model = 0x4E;      /* INTEL_FAM6_SKYLAKE_L */
+		break;
+	case 0x7E:      /* INTEL_FAM6_ICELAKE_L */
+	case 0x9D:      /* INTEL_FAM6_ICELAKE_NNPI */
+	case 0x8C:      /* INTEL_FAM6_TIGERLAKE_L */
+	case 0x8D:      /* INTEL_FAM6_TIGERLAKE */
+	case 0xA7:      /* INTEL_FAM6_ROCKETLAKE */
+	case 0x8A:      /* INTEL_FAM6_LAKEFIELD */
+	case 0x97:      /* INTEL_FAM6_ALDERLAKE */
+	case 0x9A:      /* INTEL_FAM6_ALDERLAKE_L */
+		model = 0x66;      /* INTEL_FAM6_CANNONLAKE_L */
+		break;
+	case 0x9C:      /* INTEL_FAM6_ATOM_TREMONT_L */
+		model = 0x96;      /* INTEL_FAM6_ATOM_TREMONT */
+		break;
+	case 0x6C:      /* INTEL_FAM6_ICELAKE_D */
+	case 0x8F:      /* INTEL_FAM6_SAPPHIRERAPIDS_X */
+		model = 0x6A;      /* INTEL_FAM6_ICELAKE_X */
+		break;
+	}
+
+	/* Based on probe_nhm_msrs */
+	switch (model) {
+	case 0x1E:      /* INTEL_FAM6_NEHALEM */
+	case 0x2E:      /* INTEL_FAM6_NEHALEM_EX */
+	case 0x2A:      /* INTEL_FAM6_SANDYBRIDGE */
+	case 0x2D:      /* INTEL_FAM6_SANDYBRIDGE_X */
+	case 0x3A:      /* INTEL_FAM6_IVYBRIDGE */
+	case 0x3E:      /* INTEL_FAM6_IVYBRIDGE_X */
+	case 0x3C:      /* INTEL_FAM6_HASWELL */
+	case 0x46:      /* INTEL_FAM6_HASWELL_G */
+	case 0x3F:      /* INTEL_FAM6_HASWELL_X */
+	case 0x45:      /* INTEL_FAM6_HASWELL_L */
+	case 0x3D:      /* INTEL_FAM6_BROADWELL */
+	case 0x47:      /* INTEL_FAM6_BROADWELL_G */
+	case 0x4F:      /* INTEL_FAM6_BROADWELL_X */
+	case 0x4E:      /* INTEL_FAM6_SKYLAKE_L */
+	case 0x66:      /* INTEL_FAM6_CANNONLAKE_L */
+	case 0x55:      /* INTEL_FAM6_SKYLAKE_X */
+	case 0x6A:      /* INTEL_FAM6_ICELAKE_X */
+	case 0x37:      /* INTEL_FAM6_ATOM_SILVERMONT */
+	case 0x4D:      /* INTEL_FAM6_ATOM_SILVERMONT_D */
+	case 0x4C:      /* INTEL_FAM6_ATOM_AIRMONT */
+	case 0x57:      /* INTEL_FAM6_XEON_PHI_KNL */
+	case 0x5C:      /* INTEL_FAM6_ATOM_GOLDMONT */
+	case 0x7A:      /* INTEL_FAM6_ATOM_GOLDMONT_PLUS */
+	case 0x5F:      /* INTEL_FAM6_ATOM_GOLDMONT_D */
+	case 0x96:      /* INTEL_FAM6_ATOM_TREMONT */
+	case 0x86:      /* INTEL_FAM6_ATOM_TREMONT_D */
 		break;
 	default:
 		return 0;
@@ -505,7 +549,7 @@ static void *timerthread(void *param)
 	struct sigevent sigev;
 	sigset_t sigset;
 	timer_t timer;
-	struct timespec now, next, interval, stop;
+	struct timespec now, next, interval, stop = { 0 };
 	struct itimerval itimer;
 	struct itimerspec tspec;
 	struct thread_stat *stat = par->stats;
@@ -514,8 +558,6 @@ static void *timerthread(void *param)
 	pthread_t thread;
 	unsigned long smi_now, smi_old = 0;
 
-	memset(&stop, 0, sizeof(stop));
-
 	/* if we're running in numa mode, set our memory node */
 	if (par->node != -1)
 		rt_numa_set_numa_run_on_node(par->node, par->cpu);
@@ -596,7 +638,6 @@ static void *timerthread(void *param)
 	tsnorm(&next);
 
 	if (duration) {
-		memset(&stop, 0, sizeof(stop)); /* grrr */
 		stop = now;
 		stop.tv_sec += duration;
 	}
@@ -820,10 +861,13 @@ static void display_help(int error)
 	       "-c CLOCK --clock=CLOCK     select clock\n"
 	       "                           0 = CLOCK_MONOTONIC (default)\n"
 	       "                           1 = CLOCK_REALTIME\n"
+	       "         --default-system  Don't attempt to tune the system from cyclictest.\n"
+	       "                           Power management is not suppressed.\n"
+	       "                           This might give poorer results, but will allow you\n"
+	       "                           to discover if you need to tune the system\n"
 	       "-d DIST  --distance=DIST   distance of thread intervals in us, default=500\n"
 	       "-D       --duration=TIME   specify a length for the test run.\n"
 	       "                           Append 'm', 'h', or 'd' to specify minutes, hours or days.\n"
-	       "	 --latency=PM_QOS  write PM_QOS to /dev/cpu_dma_latency\n"
 	       "-F       --fifo=<path>     create a named pipe at path and write stats to it\n"
 	       "-h       --histogram=US    dump a latency histogram to stdout after the run\n"
 	       "                           US is the max latency time to be tracked in microseconds\n"
@@ -832,10 +876,13 @@ static void display_help(int error)
 	       "	 --histfile=<path> dump the latency histogram to <path> instead of stdout\n"
 	       "-i INTV  --interval=INTV   base interval of thread in us default=1000\n"
 	       "         --json=FILENAME   write final results into FILENAME, JSON formatted\n"
-	       "-l LOOPS --loops=LOOPS     number of loops: default=0(endless)\n"
 	       "	 --laptop	   Save battery when running cyclictest\n"
 	       "			   This will give you poorer realtime results\n"
 	       "			   but will not drain your battery so quickly\n"
+	       "         --latency=PM_QOS  power management latency target value\n"
+	       "                           This value is written to /dev/cpu_dma_latency\n"
+	       "                           and affects c-states. The default is 0\n"
+	       "-l LOOPS --loops=LOOPS     number of loops: default=0(endless)\n"
 	       "         --mainaffinity=CPUSET\n"
 	       "			   Run the main thread on CPU #N. This only affects\n"
 	       "                           the main thread and not the measurement threads\n"
@@ -945,7 +992,7 @@ static char *policyname(int policy)
 
 enum option_values {
 	OPT_AFFINITY=1, OPT_BREAKTRACE, OPT_CLOCK,
-	OPT_DISTANCE, OPT_DURATION, OPT_LATENCY,
+	OPT_DEFAULT_SYSTEM, OPT_DISTANCE, OPT_DURATION, OPT_LATENCY,
 	OPT_FIFO, OPT_HISTOGRAM, OPT_HISTOFALL, OPT_HISTFILE,
 	OPT_INTERVAL, OPT_JSON, OPT_MAINAFFINITY, OPT_LOOPS, OPT_MLOCKALL,
 	OPT_REFRESH, OPT_NANOSLEEP, OPT_NSECS, OPT_OSCOPE, OPT_PRIORITY,
@@ -974,6 +1021,7 @@ static void process_options(int argc, char *argv[], int max_cpus)
 			{"aligned",          optional_argument, NULL, OPT_ALIGNED },
 			{"breaktrace",       required_argument, NULL, OPT_BREAKTRACE },
 			{"clock",            required_argument, NULL, OPT_CLOCK },
+			{"default-system",   no_argument,       NULL, OPT_DEFAULT_SYSTEM },
 			{"distance",         required_argument, NULL, OPT_DISTANCE },
 			{"duration",         required_argument, NULL, OPT_DURATION },
 			{"latency",          required_argument, NULL, OPT_LATENCY },
@@ -1011,7 +1059,7 @@ static void process_options(int argc, char *argv[], int max_cpus)
 			{"posix_timers",     no_argument,	NULL, OPT_POSIX_TIMERS },
 			{NULL, 0, NULL, 0 },
 		};
-		int c = getopt_long(argc, argv, "a::A::b:c:d:D:h:H:i:l:MNo:p:mqrRsSt::uvD:x",
+		int c = getopt_long(argc, argv, "a::A::b:c:d:D:F:h:H:i:l:MNo:p:mqrRsSt::uvD:x",
 				    long_options, &option_index);
 		if (c == -1)
 			break;
@@ -1029,7 +1077,9 @@ static void process_options(int argc, char *argv[], int max_cpus)
 			} else if (optind < argc &&
 				   (atoi(argv[optind]) ||
 				    argv[optind][0] == '0' ||
-				    argv[optind][0] == '!')) {
+				    argv[optind][0] == '!' ||
+				    argv[optind][0] == '+' ||
+				    argv[optind][0] == 'a')) {
 				parse_cpumask(argv[optind], max_cpus, &affinity_mask);
 				setaffinity = AFFINITY_SPECIFIED;
 			} else {
@@ -1038,7 +1088,7 @@ static void process_options(int argc, char *argv[], int max_cpus)
 
 			if (setaffinity == AFFINITY_SPECIFIED && !affinity_mask)
 				display_help(1);
-			if (verbose)
+			if (verbose && affinity_mask)
 				printf("Using %u cpus.\n",
 					numa_bitmask_weight(affinity_mask));
 			break;
@@ -1058,14 +1108,14 @@ static void process_options(int argc, char *argv[], int max_cpus)
 		case 'c':
 		case OPT_CLOCK:
 			clocksel = atoi(optarg); break;
-		case 'C':
+		case OPT_DEFAULT_SYSTEM:
+			power_management = 1; break;
 		case 'd':
 		case OPT_DISTANCE:
 			distance = atoi(optarg); break;
 		case 'D':
 		case OPT_DURATION:
 			duration = parse_time_string(optarg); break;
-		case 'E':
 		case 'F':
 		case OPT_FIFO:
 			use_fifo = 1;
@@ -1076,7 +1126,10 @@ static void process_options(int argc, char *argv[], int max_cpus)
 			histofall = 1; /* fall through */
 		case 'h':
 		case OPT_HISTOGRAM:
-			histogram = atoi(optarg); break;
+			histogram = atoi(optarg);
+			if (!histogram)
+				display_help(1);
+			break;
 		case OPT_HISTFILE:
 			use_histfile = 1;
 			strncpy(histfile, optarg, strnlen(optarg, MAX_PATH-1));
@@ -1815,10 +1868,7 @@ int main(int argc, char **argv)
 		printf("Online CPUs = %d\n", online_cpus);
 	}
 
-	/* Restrict the main pid to the affinity specified by the user */
-	if (main_affinity_mask != NULL) {
-		set_main_thread_affinity(main_affinity_mask);
-	} else if (affinity_mask != NULL) {
+	if (affinity_mask != NULL) {
 		set_main_thread_affinity(affinity_mask);
 		if (verbose)
 			printf("Using %u cpus.\n",
@@ -2050,7 +2100,6 @@ int main(int argc, char **argv)
 			memset(stat->values, 0, bufsize);
 			par->bufmsk = VALBUF_SIZE - 1;
 			if (smi) {
-				int bufsize = VALBUF_SIZE * sizeof(long);
 				stat->smis = threadalloc(bufsize, node);
 				if (!stat->smis)
 					goto outall;
@@ -2093,6 +2142,10 @@ int main(int argc, char **argv)
 			fatal("failed to create thread %d: %s\n", i, strerror(status));
 
 	}
+	/* Restrict the main pid to the affinity specified by the user */
+	if (main_affinity_mask != NULL)
+		set_main_thread_affinity(main_affinity_mask);
+
 	if (use_fifo) {
 		status = pthread_create(&fifo_threadid, NULL, fifothread, NULL);
 		if (status)
