@@ -1,25 +1,10 @@
+// SPDX-License-Identifier: LGPL-2.1-or-later
 /*
-   Classic Priority Inversion deadlock test case
-  
-   Copyright (C) 2006 Free Software Foundation, Inc.
-   This file is part of the GNU C Library.
-     Contributed by Clark Williams<williams@redhat.com>, 2006
-  
-   The GNU C Library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Lesser General Public License as
-   published by the Free Software Foundation; either version 2.1 of the
-   License, or (at your option) any later version.
-  
-   The GNU C Library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Lesser General Public License for more details.
-  
-   You should have received a copy of the GNU Lesser General Public
-   License along with the GNU C Library; see the file COPYING.LIB.  If not,
-   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
-
+ * Classic Priority Inversion deadlock test case
+ * Copyright (C) 2006 Free Software Foundation, Inc.
+ * This file is part of the GNU C Library.
+ * Contributed by Clark Williams<williams@redhat.com>, 2006
+ */ 
 
 /* This program tests Priority Inheritance mutexes and their ability
    to avoid Priority Inversion deadlocks
