@@ -20,6 +20,7 @@
 #include <string.h>
 #include <time.h>
 #include <utmpx.h>
+#include <inttypes.h>
 
 #include <linux/unistd.h>
 
@@ -28,9 +29,10 @@
 #include <sys/sem.h>
 #include <sys/time.h>
 #include <sys/mman.h>
+
 #include "rt-utils.h"
 #include "rt-get_cpu.h"
-#include "error.h"
+#include "rt-error.h"
 
 #define SEM_WAIT_FOR_RECEIVER 0
 #define SEM_WAIT_FOR_SENDER 1
@@ -65,7 +67,6 @@ struct params {
 	struct timeval unblocked, received, diff;
 	pthread_t threadid;
 	struct params *neighbor;
-	char error[MAX_PATH * 2];
 };
 
 static int mustfork;
@@ -94,16 +95,15 @@ void *semathread(void *param)
 	if (par->cpu != -1) {
 		CPU_ZERO(&mask);
 		CPU_SET(par->cpu, &mask);
-		if(sched_setaffinity(0, sizeof(mask), &mask) == -1)
-			snprintf(par->error, sizeof(par->error),
-			    "WARNING: Could not set CPU affinity "
-			    "to CPU #%d\n", par->cpu);
+		if (sched_setaffinity(0, sizeof(mask), &mask) == -1)
+			fatal("Could not set CPU affinity "
+			      "to CPU #%d\n", par->cpu);
 	} else {
-        	int max_cpus = sysconf(_SC_NPROCESSORS_CONF);
+		int max_cpus = sysconf(_SC_NPROCESSORS_CONF);
 
-        	if (max_cpus > 1)
-		        mustgetcpu = 1;
-	        else
+		if (max_cpus > 1)
+			mustgetcpu = 1;
+		else
 			par->cpu = 0;
 	}
 
@@ -112,8 +112,8 @@ void *semathread(void *param)
 
 	while (!par->shutdown) {
 		if (par->sender) {
- 			sb.sem_num = SEM_WAIT_FOR_SENDER;
- 			sb.sem_op = SEM_UNLOCK;
+			sb.sem_num = SEM_WAIT_FOR_SENDER;
+			sb.sem_op = SEM_UNLOCK;
 			/*
 			 * Unlocking the semaphore:
 			 *   Start of latency measurement ...
@@ -121,29 +121,29 @@ void *semathread(void *param)
 			gettimeofday(&par->unblocked, NULL);
 			semop(par->semid, &sb, 1);
 			par->samples++;
-			if(par->max_cycles && par->samples >= par->max_cycles)
+			if (par->max_cycles && par->samples >= par->max_cycles)
 				par->shutdown = 1;
 
 			if (mustgetcpu)
 				par->cpu = get_cpu();
 
- 			sb.sem_num = SEM_WAIT_FOR_RECEIVER;
- 			sb.sem_op = SEM_LOCK;
+			sb.sem_num = SEM_WAIT_FOR_RECEIVER;
+			sb.sem_op = SEM_LOCK;
 			semop(par->semid, &sb, 1);
 
- 			sb.sem_num = SEM_WAIT_FOR_SENDER;
+			sb.sem_num = SEM_WAIT_FOR_SENDER;
 			sb.sem_op = SEM_LOCK;
 			semop(par->semid, &sb, 1);
 		} else {
 			/* Receiver */
 			struct params *neighbor;
 
- 			if (wasforked)
+			if (wasforked)
 				neighbor = par + par->num_threads;
 			else
 				neighbor = par->neighbor;
 
- 			sb.sem_num = SEM_WAIT_FOR_SENDER;
+			sb.sem_num = SEM_WAIT_FOR_SENDER;
 			sb.sem_op = SEM_LOCK;
 			semop(par->semid, &sb, 1);
 
@@ -178,20 +178,19 @@ void *semathread(void *param)
 					write(tracing_enabled, "0", 1);
 					close(tracing_enabled);
 				} else
-					snprintf(par->error, sizeof(par->error),
-					    "Could not access %s\n",
-					    tracing_enabled_file);
+					fatal("Could not access %s\n",
+					      tracing_enabled_file);
 				par->shutdown = 1;
 				neighbor->shutdown = 1;
 			}
 
- 			sb.sem_num = SEM_WAIT_FOR_RECEIVER;
+			sb.sem_num = SEM_WAIT_FOR_RECEIVER;
 			sb.sem_op = SEM_UNLOCK;
 			semop(par->semid, &sb, 1);
 
 			nanosleep(&par->delay, NULL);
 
- 			sb.sem_num = SEM_WAIT_FOR_SENDER;
+			sb.sem_num = SEM_WAIT_FOR_SENDER;
 			sb.sem_op = SEM_UNLOCK;
 			semop(par->semid, &sb, 1);
 		}
@@ -211,40 +210,42 @@ void *semathread(void *param)
 
 
 union semun {
-       int		val;	/* Value for SETVAL */
-       struct semid_ds *buf;	/* Buffer for IPC_STAT, IPC_SET */
-       unsigned short  *array;	/* Array for GETALL, SETALL */
-       struct seminfo  *__buf;	/* Buffer for IPC_INFO (Linux-specific) */
+	int		val;	/* Value for SETVAL */
+	struct semid_ds *buf;	/* Buffer for IPC_STAT, IPC_SET */
+	unsigned short  *array;	/* Array for GETALL, SETALL */
+	struct seminfo  *__buf;	/* Buffer for IPC_INFO (Linux-specific) */
 };
 
 
-static void display_help(void)
+static void display_help(int error)
 {
 	printf("svsematest V %1.2f\n", VERSION);
-	puts("Usage: svsematest <options>");
-	puts("Function: test SYSV semaphore latency");
-	puts(
-	"Options:\n"
-	"-a [NUM] --affinity        run thread #N on processor #N, if possible\n"
-	"                           with NUM pin all threads to the processor NUM\n"
-	"-b USEC  --breaktrace=USEC send break trace command when latency > USEC\n"
-	"-d DIST  --distance=DIST   distance of thread intervals in us default=500\n"
-	"-f       --fork            fork new processes instead of creating threads\n"
-	"-i INTV  --interval=INTV   base interval of thread in us default=1000\n"
-	"-l LOOPS --loops=LOOPS     number of loops: default=0(endless)\n"
-	"-D       --duration=TIME   specify a length for the test run.\n"
-	"                           Append 'm', 'h', or 'd' to specify minutes, hours or days.\n"
-	"-p PRIO  --prio=PRIO       priority\n"
-	"-S       --smp             SMP testing: options -a -t and same priority\n"
-        "                           of all threads\n"
-	"-t       --threads         one thread per available processor\n"
-	"-t [NUM] --threads=NUM     number of threads:\n"
-	"                           without NUM, threads = max_cpus\n"
-	"                           without -t default = 1\n");
-	exit(1);
+	printf("Usage:\n"
+	       "svsematest <options>\n\n"
+	       "Function: test SYSV semaphore latency\n\n"
+	       "Avaiable options:\n"
+	       "-a [NUM] --affinity        run thread #N on processor #N, if possible\n"
+	       "                           with NUM pin all threads to the processor NUM\n"
+	       "-b USEC  --breaktrace=USEC send break trace command when latency > USEC\n"
+	       "-d DIST  --distance=DIST   distance of thread intervals in us default=500\n"
+	       "-D       --duration=TIME   specify a length for the test run.\n"
+	       "                           Append 'm', 'h', or 'd' to specify minutes, hours or\n"
+	       "                           days.\n"
+	       "-f [OPT] --fork[=OPT]      fork new processes instead of creating threads\n"
+	       "-i INTV  --interval=INTV   base interval of thread in us default=1000\n"
+	       "         --json=FILENAME   write final results into FILENAME, JSON formatted\n"
+	       "-l LOOPS --loops=LOOPS     number of loops: default=0(endless)\n"
+	       "-p PRIO  --prio=PRIO       priority\n"
+	       "-S       --smp             SMP testing: options -a -t and same priority\n"
+	       "                           of all threads\n"
+	       "-t       --threads         one thread per available processor\n"
+	       "-t [NUM] --threads[=NUM]   number of threads:\n"
+	       "                           without NUM, threads = max_cpus\n"
+	       "                           without -t default = 1\n"
+	       );
+	exit(error);
 }
 
-
 static int setaffinity = AFFINITY_UNSPECIFIED;
 static int affinity;
 static int priority;
@@ -255,8 +256,16 @@ static int interval = 1000;
 static int distance = 500;
 static int smp;
 static int sameprio;
+static int quiet;
+static char jsonfile[MAX_PATH];
 
-static void process_options (int argc, char *argv[])
+enum option_value {
+	OPT_AFFINITY=1, OPT_BREAKTRACE, OPT_DISTANCE, OPT_DURATION,
+	OPT_FORK, OPT_HELP, OPT_INTERVAL, OPT_JSON, OPT_LOOPS,
+	OPT_PRIORITY, OPT_QUIET, OPT_SMP, OPT_THREADS
+};
+
+static void process_options(int argc, char *argv[])
 {
 	int error = 0;
 	int max_cpus = sysconf(_SC_NPROCESSORS_CONF);
@@ -266,24 +275,27 @@ static void process_options (int argc, char *argv[])
 		int option_index = 0;
 		/** Options for getopt */
 		static struct option long_options[] = {
-			{"affinity", optional_argument, NULL, 'a'},
-			{"breaktrace", required_argument, NULL, 'b'},
-			{"distance", required_argument, NULL, 'd'},
-			{"fork", optional_argument, NULL, 'f'},
-			{"interval", required_argument, NULL, 'i'},
-			{"loops", required_argument, NULL, 'l'},
-			{"duration", required_argument, NULL, 'D'},
-			{"priority", required_argument, NULL, 'p'},
-			{"smp", no_argument, NULL, 'S'},
-			{"threads", optional_argument, NULL, 't'},
-			{"help", no_argument, NULL, '?'},
+			{"affinity",	optional_argument,	NULL, OPT_AFFINITY},
+			{"breaktrace",	required_argument,	NULL, OPT_BREAKTRACE},
+			{"distance",	required_argument,	NULL, OPT_DISTANCE},
+			{"duration",	required_argument,	NULL, OPT_DURATION},
+			{"fork",	optional_argument,	NULL, OPT_FORK},
+			{"help",	no_argument,		NULL, OPT_HELP},
+			{"interval",	required_argument,	NULL, OPT_INTERVAL},
+			{"json",	required_argument,      NULL, OPT_JSON},
+			{"loops",	required_argument,	NULL, OPT_LOOPS},
+			{"priority",	required_argument,	NULL, OPT_PRIORITY},
+			{"quiet",	no_argument,		NULL, OPT_QUIET},
+			{"smp",		no_argument,		NULL, OPT_SMP},
+			{"threads",	optional_argument,	NULL, OPT_THREADS},
 			{NULL, 0, NULL, 0}
 		};
-		int c = getopt_long (argc, argv, "a::b:d:f::i:l:D:p:St::h",
+		int c = getopt_long (argc, argv, "a::b:d:D:f::hi:l:p:qSt::",
 			long_options, &option_index);
 		if (c == -1)
 			break;
 		switch (c) {
+		case OPT_AFFINITY:
 		case 'a':
 			if (smp) {
 				warn("-a ignored due to --smp\n");
@@ -292,15 +304,26 @@ static void process_options (int argc, char *argv[])
 			if (optarg != NULL) {
 				affinity = atoi(optarg);
 				setaffinity = AFFINITY_SPECIFIED;
-			} else if (optind<argc && atoi(argv[optind])) {
+			} else if (optind < argc && atoi(argv[optind])) {
 				affinity = atoi(argv[optind]);
 				setaffinity = AFFINITY_SPECIFIED;
 			} else {
 				setaffinity = AFFINITY_USEALL;
 			}
 			break;
-		case 'b': thistracelimit = atoi(optarg); break;
-		case 'd': distance = atoi(optarg); break;
+		case OPT_BREAKTRACE:
+		case 'b':
+			thistracelimit = atoi(optarg);
+			break;
+		case OPT_DISTANCE:
+		case 'd':
+			distance = atoi(optarg);
+			break;
+		case OPT_DURATION:
+		case 'D':
+			duration = parse_time_string(optarg);
+			break;
+		case OPT_FORK:
 		case 'f':
 			if (optarg != NULL) {
 				wasforked = 1;
@@ -312,15 +335,36 @@ static void process_options (int argc, char *argv[])
 			} else
 				mustfork = 1;
 			break;
-		case 'i': interval = atoi(optarg); break;
-		case 'l': max_cycles = atoi(optarg); break;
-		case 'D': duration = parse_time_string(optarg); break;
-		case 'p': priority = atoi(optarg); break;
+		case OPT_HELP:
+		case 'h':
+			display_help(0);
+			break;
+		case OPT_INTERVAL:
+		case 'i':
+			interval = atoi(optarg);
+			break;
+		case OPT_JSON:
+			strncpy(jsonfile, optarg, strnlen(optarg, MAX_PATH-1));
+			break;
+		case OPT_LOOPS:
+		case 'l':
+			max_cycles = atoi(optarg);
+			break;
+		case OPT_PRIORITY:
+		case 'p':
+			priority = atoi(optarg);
+			break;
+		case OPT_QUIET:
+		case 'q':
+			quiet = 1;
+			break;
+		case OPT_SMP:
 		case 'S':
 			smp = 1;
 			num_threads = max_cpus;
 			setaffinity = AFFINITY_USEALL;
 			break;
+		case OPT_THREADS:
 		case 't':
 			if (smp) {
 				warn("-t ignored due to --smp\n");
@@ -328,13 +372,14 @@ static void process_options (int argc, char *argv[])
 			}
 			if (optarg != NULL)
 				num_threads = atoi(optarg);
-			else if (optind<argc && atoi(argv[optind]))
+			else if (optind < argc && atoi(argv[optind]))
 				num_threads = atoi(argv[optind]);
 			else
 				num_threads = max_cpus;
 			break;
-		case 'h': error = 1; break;
-		case '?': error = 1; break;
+		default:
+			display_help(1);
+			break;
 		}
 	}
 
@@ -365,7 +410,7 @@ static void process_options (int argc, char *argv[])
 		tracelimit = thistracelimit;
 	}
 	if (error)
-		display_help ();
+		display_help(error);
 }
 
 
@@ -376,6 +421,80 @@ static void sighand(int sig)
 	mustshutdown = 1;
 }
 
+struct params_stats {
+	struct params *receiver;
+	struct params *sender;
+};
+
+static void write_stats(FILE *f, void *data)
+{
+	struct params_stats *ps = data;
+	struct params *s, *r;
+	unsigned int i;
+
+	fprintf(f, "  \"num_threads\": %d,\n", num_threads);
+	fprintf(f, "  \"thread\": {\n");
+	for (i = 0; i < num_threads; i++) {
+		s = &ps->sender[i];
+		r = &ps->receiver[i];
+		fprintf(f, "    \"%u\": {\n", i);
+		fprintf(f, "      \"sender\": {\n");
+		fprintf(f, "        \"cpu\": %d,\n", s->cpu);
+		fprintf(f, "        \"priority\": %d,\n", s->priority);
+		fprintf(f, "        \"samples\": %d,\n", s->samples);
+		fprintf(f, "        \"interval\": %ld\n", r->delay.tv_nsec/1000);
+		fprintf(f, "      },\n");
+		fprintf(f, "      \"receiver\": {\n");
+		fprintf(f, "        \"cpu\": %d,\n", r->cpu);
+		fprintf(f, "        \"priority\": %d,\n", r->priority);
+		fprintf(f, "        \"min\": %d,\n", r->mindiff);
+		fprintf(f, "        \"avg\": %.2f,\n", r->sumdiff/r->samples);
+		fprintf(f, "        \"max\": %d\n", r->maxdiff);
+		fprintf(f, "      }\n");
+		fprintf(f, "    }%s\n", i == num_threads - 1 ? "" : ",");
+	}
+	fprintf(f, "  }\n");
+}
+
+static void print_stat(FILE *fp, struct params *receiver, struct params *sender,
+		       int verbose, int quiet)
+{
+	int i;
+
+	if (quiet)
+		return;
+
+	for (i = 0; i < num_threads; i++) {
+		int receiver_pid, sender_pid;
+
+		if (mustfork) {
+			receiver_pid = receiver[i].pid;
+			sender_pid = sender[i].pid;
+		} else {
+			receiver_pid = receiver[i].tid;
+			sender_pid = sender[i].tid;
+		}
+		printf("#%1d: ID%d, P%d, CPU%d, I%ld; #%1d: ID%d, P%d, CPU%d, Cycles %d\n",
+			i*2, receiver_pid, receiver[i].priority,
+			receiver[i].cpu, receiver[i].delay.tv_nsec /
+			1000, i*2+1, sender_pid, sender[i].priority,
+			sender[i].cpu, sender[i].samples);
+	}
+
+	for (i = 0; i < num_threads; i++) {
+		if (receiver[i].mindiff == -1)
+			printf("#%d -> #%d (not yet ready)\n",
+				i*2+1, i*2);
+		else
+			printf("#%d -> #%d, Min %4d, Cur %4d, Avg %4d, Max %4d\n",
+				i*2+1, i*2, receiver[i].mindiff,
+				(int) receiver[i].diff.tv_usec,
+				(int) ((receiver[i].sumdiff /
+						receiver[i].samples) + 0.5),
+				receiver[i].maxdiff);
+	}
+}
+
 int main(int argc, char *argv[])
 {
 	char *myfile;
@@ -383,7 +502,7 @@ int main(int argc, char *argv[])
 	int max_cpus = sysconf(_SC_NPROCESSORS_CONF);
 	int oldsamples = 1;
 	key_t key;
-        union semun args;
+	union semun args;
 	struct params *receiver = NULL;
 	struct params *sender = NULL;
 	sigset_t sigset;
@@ -395,6 +514,7 @@ int main(int argc, char *argv[])
 	if (myfile == NULL)
 		myfile = argv[0];
 
+	rt_init(argc, argv);
 	process_options(argc, argv);
 
 	if (check_privs())
@@ -418,7 +538,7 @@ int main(int argc, char *argv[])
 		totalsize = num_threads * sizeof(struct params) * 2;
 
 		shm_unlink("/sigwaittest");
-  		shmem = shm_open("/sigwaittest", O_CREAT|O_EXCL|O_RDWR,
+		shmem = shm_open("/sigwaittest", O_CREAT|O_EXCL|O_RDWR,
 		    S_IRUSR|S_IWUSR);
 		if (shmem < 0) {
 			fprintf(stderr, "Could not create shared memory\n");
@@ -605,57 +725,15 @@ int main(int argc, char *argv[])
 	}
 
 	while (!mustshutdown) {
-		int printed;
-		int errorlines = 0;
-
 		for (i = 0; i < num_threads; i++)
 			mustshutdown |= receiver[i].shutdown |
 			    sender[i].shutdown;
 
 		if (receiver[0].samples > oldsamples || mustshutdown) {
-			for (i = 0; i < num_threads; i++) {
-				int receiver_pid, sender_pid;
-
-				if (mustfork) {
-					receiver_pid = receiver[i].pid;
-					sender_pid = sender[i].pid;
-				} else {
-					receiver_pid = receiver[i].tid;
-					sender_pid = sender[i].tid;
-				}
-				printf("#%1d: ID%d, P%d, CPU%d, I%ld; #%1d: "
-				    "ID%d, P%d, CPU%d, Cycles %d\n",
-				    i*2, receiver_pid, receiver[i].priority,
-				    receiver[i].cpu, receiver[i].delay.tv_nsec /
-				    1000, i*2+1, sender_pid, sender[i].priority,
-				    sender[i].cpu, sender[i].samples);
-			}
-			for (i = 0; i < num_threads; i++) {
-				if (receiver[i].mindiff == -1)
-					printf("#%d -> #%d (not yet ready)\n",
-					    i*2+1, i*2);
-				else
-					printf("#%d -> #%d, Min %4d, Cur %4d, "
-					    "Avg %4d, Max %4d\n",
-					    i*2+1, i*2, receiver[i].mindiff,
-					    (int) receiver[i].diff.tv_usec,
-					    (int) ((receiver[i].sumdiff /
-					    receiver[i].samples) + 0.5),
-					    receiver[i].maxdiff);
-				if (receiver[i].error[0] != '\0') {
-					printf("%s", receiver[i].error);
-					receiver[i].error[0] = '\0';
-					errorlines++;
-				}
-				if (sender[i].error[0] != '\0') {
-					printf("%s", sender[i].error);
-					sender[i].error[0] = '\0';
-					errorlines++;
-				}
-			}
-			printed = 1;
-		} else
-			printed = 0;
+			print_stat(stdout, receiver, sender, 0, quiet);
+			if (!quiet)
+				printf("\033[%dA", num_threads*2);
+		}
 
 		sigemptyset(&sigset);
 		sigaddset(&sigset, SIGTERM);
@@ -667,11 +745,13 @@ int main(int argc, char *argv[])
 
 		sigemptyset(&sigset);
 		pthread_sigmask(SIG_SETMASK, &sigset, NULL);
-
-		if (printed && !mustshutdown)
-			printf("\033[%dA", num_threads*2 + errorlines);
 	}
 
+	if (!quiet)
+		printf("\033[%dB", num_threads*2 + 2);
+	else
+		print_stat(stdout, receiver, sender, 0, 0);
+
 	for (i = 0; i < num_threads; i++) {
 		receiver[i].shutdown = 1;
 		sender[i].shutdown = 1;
@@ -693,11 +773,19 @@ int main(int argc, char *argv[])
 		}
 	}
 
-	nosem:
+	if (strlen(jsonfile) != 0) {
+		struct params_stats ps = {
+			.receiver = receiver,
+			.sender = sender,
+		};
+		rt_write_json(jsonfile, 0, write_stats, &ps);
+	}
+
+nosem:
 	for (i = 0; i < num_threads; i++)
 		semctl(receiver[i].semid, -1, IPC_RMID);
 
- 	nomem:
+nomem:
 	if (mustfork) {
 		munmap(param, totalsize);
 		shm_unlink("/sigwaittest");
